/*
 * @(#)HelloAglet.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package examples.hello;

import aglet.*;
import aglet.event.*;
import aglet.util.*;

import ibm.aglets.util.*;

import java.lang.InterruptedException;
import java.io.Externalizable;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.IOException;
import java.net.*;
import java.awt.*;

import java.util.Enumeration;

/**
 * HelloAglet is an mobile aglet that goes to a remote host
 * to say "Hello World" and then returns home and dies.
 *
 * @version     1.00	$Date: 7/08/97 1:32p $
 * @author	Danny B. Lange
 * @author	Mitsuru Oshima
 */
public class HelloAglet extends Aglet {

    /*
     * UI to interact with a User
     * this will be automatically disposed when the aglet is disposed
     */
    transient Frame my_dialog = null;

    /*
     *
     */
    String message = "Hello World!";

    /*
     *
     */
    String home = null;

    /*
     * Itinerary
     */
    SimpleItinerary itinerary = null;


    /*
     * Initializes the aglet. Only called the very first time this 
     * aglet is created. 
     */
    public void onCreation(Object init) {
	itinerary = new SimpleItinerary(this);

	// Initialize the variables.
	home = getAgletContext().getHostingURL().toString();
    }

    /*
     * Go to the destination and say hello!
     */
    public void go(Message msg) {
	URL dest = (URL)msg.getArg();
	try {
	    itinerary.go(dest.toString(), "sayHello");
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    /**
     * Dispatch the aglet to the destination.
     * @param destination a url which specifies the destination
     * @exception when the aglet is in the invalid state
     */
    public synchronized void goDestination(String destination) {
	try {
	    itinerary.go(destination, "sayHello");
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    /*
     * Handles the message
     * @param msg the message sent
     */
    public boolean handleMessage(Message msg) {
	if (msg.sameKind("atHome")) {
	    atHome(msg);
	} else if (msg.sameKind("sayHello")) {
	    sayHello(msg);
	} else if (msg.sameKind("dialog")) {
	    dialog(msg);
	} else {
	    return false;
	}
	return true;
    }

    /*
     *
     */
    public void atHome(Message msg) {
	setText("I'm back.");
	waitMessage(2 * 1000);
	dispose();
    }

    /*
     * Say hello!
     */
    public void sayHello(Message msg) {
	// greetings
	setText(message);

	waitMessage(5 * 1000);

	try {
	    setText("I'll go back to.. " + home);
	    waitMessage(1000);
	    itinerary.go(home, "atHome");
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    /**
     * Creates the dialog window. This has the reference to the instance
     * of the Dialog to avoid opening multiple dialog windows.
     */
    public void dialog(Message msg) {
	if (my_dialog == null) {
	    my_dialog = new MyDialog(this);
	    my_dialog.pack();
	    my_dialog.resize(my_dialog.preferredSize());
	}
	my_dialog.show();
    }
}


/*
 * MyDialog class is the window to be opened when the dialog required.
 * This is NOT a subclass of Dialog.
 */
class MyDialog extends Frame {

    /*
     * The aglet a user interacts with.
     */
    private HelloAglet aglet  = null;

    /*
     * UI Components
     */
    private AddressChooser dest = new AddressChooser();
    private TextField msg       = new TextField(15);
    private Button go           = new Button("GO!");
    private Button close        = new Button("CLOSE");

    /*
     * Constructs the dialog window
     * @param aglet The aglet the user interacts with.
     */
    MyDialog(HelloAglet aglet) {
	this.aglet = aglet;
	layoutComponents();
    }

    /*
     * Layouts all components
     */
    private void layoutComponents() {
	msg.setText(aglet.message);

	// Layouts components
	GridBagLayout grid = new GridBagLayout();
	GridBagConstraints cns = new GridBagConstraints();
	setLayout(grid);

	cns.weightx = 0.5;
	cns.ipadx = cns.ipady = 5;
	cns.fill = GridBagConstraints.HORIZONTAL;
	cns.insets = new Insets(5,5,5,5);

	cns.weightx = 1.0;
	cns.gridwidth = GridBagConstraints.REMAINDER;
	grid.setConstraints(dest, cns);
	add(dest);

	cns.gridwidth = GridBagConstraints.REMAINDER;
	cns.fill = GridBagConstraints.BOTH;
	cns.weightx = 1.0;
	cns.weighty = 1.0;
	cns.gridheight = 2;
	grid.setConstraints(msg, cns);
	add(msg);

	cns.weighty = 0.0;
	cns.fill = GridBagConstraints.NONE;
	cns.gridheight = 1;

	Panel p = new Panel();

	grid.setConstraints(p, cns);
	add(p);
	p.setLayout(new FlowLayout());
	p.add(go);
	p.add(close);
    }

    /**
     * Handles the events
     * @param ev the event to be handled
     */
    public boolean handleEvent(Event ev) {
	if (ev.id == Event.WINDOW_DESTROY) {
	    hide();
	    return true;
	}
	return super.handleEvent(ev);
    }

    /**
     * Handles the action
     * @param ev the event to be handled
     * @param arg the extra argument
     */
    public boolean action(Event ev, Object obj) {
	if (ev.target == go) {
	    aglet.message = msg.getText();
	    aglet.goDestination(dest.getAddress());
	} else if (ev.target == close) {
	    hide();
	} else {
	    return false;
	}
	return true;
    }
}
