/*
 * @(#)WebServerAglet.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package examples.http;

import aglet.*;
import java.io.*;
import java.net.URL;

/**
 * WebServerAglet is an aglet which behaves like WebServer.
 * Please enable the HTTP messaging feature in the configuration panel
 * Options -> Network Cofiguration -> Others.
 * If the aglet is successfully created, please try
 * <pre>
 * http://aglet.server:434/aglets/default/test/index.html
 * </pre>
 * in your web browser. The port number have to be same number on which
 * the aglet server is running. (434 by default)
 *
 * @version     1.00	$Date: 7/10/97 1:43p $
 * @author	Mitsuru Oshima
 */
public class WebServerAglet extends Aglet {
    String indexPage = null;

    public void onCreation(Object init) {

	//
	// this is just a convension.
	//
	getAgletContext().setProperty("name.test", getAgletID());

	//
	// Creating pages in advance.
	//
	indexPage();

	//
	// Accepts http requests concurrently.
	//
	getMessageManager().setPriority("index.html",
					MessageManager.NOT_QUEUED);
	getMessageManager().setPriority("next.html",
					MessageManager.NOT_QUEUED);
    }

    public boolean handleMessage(Message m) {
	//
	// This is just a temporary solution.
	//
	Object o = m.getArg("cgi-response");
	PrintStream p = null;

	if (o != null && o instanceof OutputStream) {
	    return handleHttpRequest(m, new PrintStream((OutputStream)o));
	}
	return true;
    }

    //
    // If it's http request.
    //
    boolean handleHttpRequest(Message m, PrintStream p) {

	if (m.sameKind("index.html") || m.sameKind("")) {
	    p.println(indexPage);
	} else if (m.sameKind("next.html")) {
	    p.println("<HTML><BODY>");
	    p.println("<h1> Here is a list of proxies in <tt>" +
		      getAgletContext().getHostingURL() + "</tt></h1>");
	    AgletProxy[] proxies =
		    getAgletContext().getAgletProxies(ACTIVE|INACTIVE);
	    p.println("<PRE>");
	    for(int i=0; i<proxies.length; i++) {
		try {
		    p.println( proxies[i].getAgletInfo().toString() );
		} catch (Exception ex) {
		    ex.printStackTrace(p);
		}
	    }
	    p.println("</PRE>");
	    p.println("</BODY></HTML>");
	    m.sendReply("text/html");
	} else if (m.sameKind("go")) {
	    System.out.println( (String)m.getArg("location") );
	    String l = (String)m.getArg("location");
	    if (l.startsWith("atp:")) {
		l = l.substring(4);
		if (l.indexOf(':')<0) {
		    p.println("<html> <body> <H1>");
		    p.println("Please specify port number. default = 434");
		    p.println("</H1></body></html>");
		    m.sendReply("text/html");
		    return true;
		}
	    }

	    try {
		p.println("<html> <body> <H1> Moving to...! </H1>");
		String contextName = getAgletContext().getName();

		if (contextName.equals("")) {
		    contextName = "default";
		}

		p.println("<a href= \"http:" + l + "/aglets/" +
			  contextName + "/" + getAgletID() +"/index.html\" TARGET=_top> atp:" + l + " </a>");
		p.println("Click above link to trace me! <BR>");
		p.println("</body> </html>");
		m.sendReply("text/html");

		dispatch(new java.net.URL("atp:" + l));

	    } catch (IOException ex) {
		ex.printStackTrace();
	    } catch (RequestRefusedException ex) {
		ex.printStackTrace();
	    }
	} else return false;
	return true;
    }

    /*
     * index page
     */
    void indexPage() {
	StringBuffer b = new StringBuffer();
	b.append("<HTML><BODY>");
	b.append("<H1> Welcome to WebServerAglet! </H1>");
	b.append("<a href=next.html> List Proxies </a> <BR>");
	b.append("<FORM METHOD=GET ACTION=go>");
	b.append("<INPUT NAME=location VALUE=\"atp://your.host\">");
	b.append("<INPUT TYPE=submit VALUE=GO!>");
	b.append("</BODY></HTML>");
	indexPage = b.toString();
    }
}

