/*
 * @(#)TalkMaster.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 *
 */

package examples.manual;

import aglet.a4compatibility.*;
import aglet.*;
import java.io.Externalizable;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.IOException;
import java.net.URL;

/**
 * This is an example of agnet which uses OldAPI and ManualSerialization.
 * This is not implementing Externalizable because this is stationaly. 
 *
 * @version     1.00    $Date: 7/09/97 5:41p $
 * @author      Mitsuru Oshima
 * @see examples.manual.TalkSlave
 */
public class TalkMaster extends OldAglet {

    //
    // ManualSerialization cannot serialize AgletProxy objects
    //
    transient AgletProxy remoteProxy = null;

    String name = "Unknown";

    public TalkMaster() {
	// do nothing
    }

    TalkWindow window = null;

    public void onCreation(Object o) {
	window = new TalkWindow(this);
	window.pack();
	window.show();
	try {
	    name = (String) getAgletContext().getProperty("aglets.user.name", "Unknown");
	} catch (Exception ex) {
	}
	super.onCreation(o);
    }

    public void onDisposing() {
	if (window != null) {
	    window.dispose();
	    window = null;
	}
    }

    public void onReverting(URL remoteURL) {
	throw new SecurityException();
    }
    public void onDispatching(URL dest) {
	throw new SecurityException();
    }

    public void dispatchSlave(String dest) {
	try {
	    if (remoteProxy != null) {
		remoteProxy.sendMessage(new Message("bye"));
	    }

	    AgletContext context = getAgletContext();

	    AgletProxy proxy = context.createAglet(null, "examples.manual.TalkSlave", null);
	    proxy.sendMessage(new Message("id", getIdentifier()));

	    URL url = new URL(dest);
	    remoteProxy = proxy.dispatch(url);

	} catch (InvalidAgletException ex) {
	    ex.printStackTrace();
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    void sendText(String text) {
	try {
	    if (remoteProxy != null) {
		remoteProxy.sendMessage(new Message("text",
						    name + " : " + text));
	    }
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    public boolean handleMessage(Message msg) {
	if ("dialog".equals(msg.kind)) {
	    window.show();
	} else if ("text".equals(msg.kind)) {
	    if (window.isVisible() == false) {
		window.show();
	    }
	    window.appendText((String)msg.getArg());
	    return true;
	}
	return false;
    }

    private void print(String m) {
   	System.out.println("Sender: " + m);
    }
}
