/*
 * @(#)Stationary
 *
 */

package examples.meeting;

import aglet.*;
import aglet.event.*;
import ibm.aglets.patterns.*;
import java.io.IOException;
import java.net.URL;
import ibm.aglets.util.*;
import java.util.*;

/**
 *
 * A stationary aglet ("Stationary C") creates two mobile aglets 
 * ("Traveller A" and "Traveller B"). They meet together with 
 * "Stationary C" after returning back from thier trips.
 *
 * @version     1.00    97/03/04
 * @author      Yariv Aridor
 */

public class Stationary extends Aglet implements MobilityListener, PersistencyListener {

    private Meeting _meeting = null;

    public Stationary() {
	// do nothing
    }

    public void onCreation(Object o) {
	print("OnCreation");
	setText("Don't try to move me!!");
	addMobilityListener(this);
    }

    public void onDisposing() {
	print("disposing");
    }

    public void onArrival (MobilityEvent ev) {
    }

    public void onReverting(MobilityEvent ev) {
	throw new SecurityException();
    }


    public void onDispatching(MobilityEvent ev) {
	throw new SecurityException("Don't try to move me!!");
    }


    public void onDeactivating(PersistencyEvent event) {
	throw new SecurityException();
    }

    public void onActivation(PersistencyEvent event) {
    }


    private Hashtable aglets = new Hashtable();

    public void run() {
	AgletProxy p = null;
	try {
	    aglets.put(getAgletID(),"Stationary C");
	    Vector c = new Vector();
	    c.addElement(getAgletID());
	    _meeting = new Meeting(getAgletContext().getHostingURL(),c);
	    _meeting.setAttachedInfo(aglets);

	    synchronized (_meeting) {
		p = getAgletContext().createAglet(null,"examples.meeting.Traveller",_meeting);	
		aglets.put(p.getAgletID(),"Traveller A");

		p = getAgletContext().createAglet(null,"examples.meeting.Traveller",_meeting);    
		aglets.put(p.getAgletID(),"Traveller B");
	    }
	    print("I did not plan to meet with anyone but...");
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }	

    private String map (AgletID id) {
	return (String)aglets.get(id);
    }

    private Vector toVector(Enumeration e) {
	Vector v = new Vector();
	for (;e.hasMoreElements();) {
	    v.addElement(e.nextElement());
	}
	return v;
    }


    private void print(String m)  {
   	try { 
	    System.out.println(map(getAgletID()) + ": " + m);
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }



}

