/*
 * @(#)Traveller
 *
 */

package examples.meeting;

import aglet.*;
import aglet.event.*;
import ibm.aglets.patterns.*;

import java.io.IOException;
import java.net.URL;
import ibm.aglets.util.*;
import java.util.*;


/**
 * A Traveller aglets can be dispatched to different hosts. Upon return
 * to its origin, it meets with its creator.
 *
 * @version     1.00    97/03/04
 * @author      Yariv Aridor
 */
public class Traveller extends Aglet implements MobilityListener,PersistencyListener {

    private static final int AT_HOME=0;
    private static final int OUT=1;
    private static final int BACK_HOME=2;

    private URL _origin = null;
    private Meeting _meeting = null;
    private int _state = AT_HOME;
    private Hashtable aglets = null;

    private int count = 0;

    public Traveller() {
	// do nothing
    }

    public void onCreation(Object o) {
	_meeting = (Meeting)o;
	aglets = (Hashtable)_meeting.getAttachedInfo();
	setText("dispatch me somewhere (I will return)");
	addMobilityListener(this);
    }

    public void onDisposing() {
	print("Disposed");
    }

    public void onArrival(MobilityEvent ev) {
	print("OnArrival");
    }


    public void onReverting(MobilityEvent ev) {
	throw new SecurityException();
    }


    public void onDispatching(MobilityEvent ev) {
   	print("Dispatching to " + ev.getLocation() + ".");
	_state = ((_state==OUT) ? BACK_HOME : OUT);
    }


    public void onDeactivating(PersistencyEvent event) {
	throw new SecurityException("not allowed");
    }

    public void onActivation(PersistencyEvent event) {
    }


    public void run() {
	try {
	    switch (_state) {
	    case OUT: 
		dispatch(_meeting.getPlace());
		break;
	    case BACK_HOME:
		setText("I'm back");
		Enumeration e = _meeting.ready(this);
		Vector v = toVector(e);
		print("I met with " + v);
		break;
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }	

    public boolean handleMessage(Message msg) {
	try {
	    if (_meeting.getId().equals(msg.kind)) {
		meet((AgletID)(msg.getArg()));
		msg.sendReply(getIdentifier());
		return true;
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}
	return false;
    }


    private void meet (AgletID id)  {
	print ("I met with " + map(id));
    }

    private String map (AgletID id) {
	return (String)aglets.get(id);
    }


    private Vector toVector(Enumeration e) {
	Vector v = new Vector();
	for (;e.hasMoreElements();) {
	    v.addElement(map((AgletID)e.nextElement()));
	}
	return v;
    }


    private void print(String m) {
	try {
	    System.out.println(map(getAgletID()) + ": " + m);
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

}


