/*
 * @(#)FingerInfo.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package ibm.aglets.samples;

import java.util.Date;
import java.io.*;

/**
 * The FingerInfo class defines the Finger Information.
 *
 * @see Finger
 * @version     1.00    96/12/28
 * @author      Danny B. Lange
 * @author      Yariv Aridor
 */

final class FingerInfo implements Serializable {
   // Private variables
   //

   private String _hostName;
   private String _userName;
   private String _orgName;
   private String _eMail;
   private String _registered;
   private Date _localTime;

// Public methods
//
   /**
    * A constructor. 
    * @param kind specify norman notification, expiration, or error exception
    * @param notifier is URL for identifying the sender aglet.
    * @param number is used when Notifier stays at remote server to send
    * multiple instances of Notification. It is used for numbering the message.
    * @param message an argument Object; contents of the message.
    */
   FingerInfo(String hostName,  String userName, String orgName, 
                     String eMail, String registered, Date localTime) {
      _hostName = hostName;
      _userName = userName;
      _orgName = orgName;
      _eMail = eMail;
      _registered = registered;
      _localTime = localTime;
   }

   String getHostName() {
      return _hostName;
   }

   String getUserName() {
      return _userName;
   }

   String getOrgName() {
      return _orgName;
   }

   Date getLocalTime() {
      return _localTime;
   }

   String getRegistered () {
   	return _registered;
   }

   String getEmail() {
      return _eMail;
   }

   String toTextBlock () { 
     String str = "Host Name: " + _hostName + "\n" +
                  "User Name: " + _userName + "\n" +
                  "Organization Name: " + _orgName + "\n" +
                  "E-mail Address: " + _eMail + "\n"  + 
                  "Registered: " + _registered + "\n" +
                  "Local Time: " + _localTime.toString();

     return str; 
   }
}
