/*
 * @(#)FingerSlave.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package ibm.aglets.samples;

import aglet.*;
import ibm.aglets.patterns.*;

import java.util.Date;
import java.util.Vector;
import java.util.Enumeration;

/**
 * FingerSlave is a aglet of Slave pattern that moves to a remote
 * site and backs to the original site with information.
 *
 * @see Slave
 * @version     1.00    96/12/28
 * @author      Danny B. Lange
 * @author      Yariv Aridor
 *
 */

public class FingerSlave extends Slave {

   protected  synchronized void initializeJob () {
      RESULT = null;
   }
   
   protected synchronized void doJob() throws AgletException {
      RESULT = getLocalInfo();
   }

   private Object getLocalInfo() throws AgletException {
      AgletContext ac = getAgletContext();
      String hostname; 

      if (ac.getHostingURL() == null) {
	  hostname = "Unknown"; 
      } else {
          hostname=ac.getHostingURL().getHost().toString();        
      } 

      FingerInfo info = 
         new FingerInfo (hostname,
                (String)ac.getProperty("aglets.user.name", "Unknown"),
                (String)ac.getProperty("aglets.user.organization", "Unknown"),
                (String)ac.getProperty("aglets.user.email", "Unknown"),
			 (String)ac.getProperty("aglets.registered", "Unknown"),
                (new Date())
                );
      
      return info;
   } 
}
