/*
 * @(#)SampleAglet.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package ibm.aglets.samples;

import aglet.*;
import aglet.a4compatibility.*;

import ibm.aglets.patterns.*;
import ibm.aglets.util.*;
import java.util.Vector;
import java.net.*;
import java.util.Properties;
import java.util.Enumeration;
import java.util.Date;


/**
 * This class abstracts a stationary master aglet for the verious 
 * samples in this package. 
 *
 * @version     1.00    96/12/28
 * @author      Yariv Aridor
 */

public abstract class SampleAglet extends OldAglet {
	
   // The main interaction window. 
   protected SampleWindow _msw = null;

   //--  Makes this aglet immobile  
   //
   public synchronized void onDispatching(URL URL) {
      // I will shout if you try to move me!
      throw new SecurityException("Don't ever try to move me!");
   } 

   public void onCreation(Object o) {
       try {
	   subscribeMessage("updateWindow");
       } catch (Exception e) {
	   //-- not yet handled
       }
      super.onCreation(o);
   }

   public void onReverting(URL remoteURL) {
       throw new SecurityException();
   }


   public void onDeactivating(Date date) {
       _msw.hide();
   }

   public void onActivation(Date date) {
       _msw.show();
   }

   //-- Disposes the interaction window.
   //
   public synchronized void onDisposing() {
      // Removes any windows if disposed.
      if (_msw != null)
         _msw.dispose();
   }      
   
   //-- Shows or brings to top the main window.
   //
   protected synchronized void Dialog() {
      if (_msw != null) 
         _msw.show();        
   }


   //-- Handler for messages
   //
   public boolean handleMessage(Message msg) {
      try {
	   if ("updateWindow".equals(msg.kind)) 
	      updateWindow();
	   else if ("result".equals(msg.kind)) 
		   callback(msg.getArg());
	   else if ("error".equals(msg.kind)) 
		   inError((String)(msg.getArg()));
	   else
	        return false;
	   return false;
      } catch (Exception e) {
	  // -- not yet handled
      }
      return false;
   }


   //-- Callback methods for messages
   //
   protected synchronized void callback (Object obj) {
   }


   protected synchronized void inError(Object message) {
      setTheMessage((String)message);
   }


   protected synchronized void updateWindow () throws Exception  {
      AgletContext ctx = getAgletContext();
      _msw.setLocation((String)ctx.getProperty("location",_msw.getLocation()));
      _msw.setFilename((String)ctx.getProperty("filename",_msw.getFilename()));
   }


   //--  Sets the message line in the interaction window.
   //
   protected synchronized void setTheMessage(String text) {
      super.setText(text);
      if (_msw != null)
         _msw.setMessage(text);
   }


   //--  Clears the output areas in the main window.
   //
   protected void resetTheWindow () {
      if (_msw != null) {
         //_msw.clearMessage();
         _msw.clearResult();
      }
   }

   
   //--  Entry point for the aglet's own thread.
   //
   public void run() { 
      setText("Starting...");
   }

   //-- callback function for the "go" bottom.
   //
   protected void go(URL url) {
     String _target = url.toString(); 

     if  (url.getHost().equals(""))  // in case the URL address
         _target="localhost";        // does not start with "atp://"

     resetTheWindow();
     setTheMessage("going to: " + _target);

//     try {  
	 getAgletContext().setProperty("location", (String)_msw.getLocation());
	 getAgletContext().setProperty("filename",(String) _msw.getFilename());
	 getAgletContext().multicastMessage(new Message("updateWindow"));
//     } catch (AgletException ae) {
//     	   inError(ae.getMessage());
//     }
   }   

}

