/*
 * @(#)Searcher.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package ibm.aglets.samples;

import aglet.*;

import ibm.aglets.patterns.*;
import ibm.aglets.util.*;

import java.net.URL;
import java.net.MalformedURLException;
import java.io.IOException;

import java.util.Vector;
import java.util.Properties;
import java.util.Enumeration;

/**
 * Class Searcher is used to retrieve local user information 
 * from a remote aglet server. Given a list of URLs it will dispatch 
 * a slave (the <tt>SearchSlave </tt> class) to search for files whose 
 * names are matched with a specific string. The slave will return 
 * with the information to be displayed by the master. 
 *
 * @see SearcherWindow
 * @see SearcherSlave
 * @version     1.00     96/12/28
 * @author      Yoshiaki Mima
 * @author      Yariv Aridor
 */

public class Searcher extends SampleAglet  {

   //-- name of the slave's class
   private final static String SlaveClassName
              = "ibm.aglets.samples.SearcherSlave";

   //-- the result of the search
   private Vector _fileList = null;


   public void onCreation (Object o) { 
      super.onCreation(o);
      try { _msw = new SearcherWindow(this);
            updateWindow();
      } catch (Exception e) {
          inError(e.getMessage()); 
      }
   }


   //-- Message handler.
   public boolean handleMessage(Message msg) {
     if ("status report".equals(msg.kind)) { 
	statusReport((String)(msg.getArg()));    
	return false;
     }
     else 
        return super.handleMessage(msg);
   }

   /**
    * The master's callback. The entry point for the returning slave.
    * This method is a part of the Master-Slave usage pattern.
    * @param s the slave.
    * @param arg the result of the slave's work. 
    */
   protected synchronized void callback(Object arg) {
      String ni;

      // Slave is back...
      setTheMessage("Searcher has returned");

      _fileList = (Vector)arg;

      for (int i = 0; i < _fileList.size(); i++) {
         SearcherInfo si= (SearcherInfo)(_fileList.elementAt(i));
         Vector v  = si.getFileList();
	 String URLString = si.getURLString();
         for (int j=0; j < v.size(); j++) { 
             ((SearcherWindow)_msw).addResultList(
                SearcherInfo.getFullPathName(URLString,
                                             (String)(v.elementAt(j))));
         }
      }
   }


   protected synchronized void statusReport(String text) {
      setTheMessage(text);
   }


   //-- display the files found for a specific URL. 
   //
   synchronized void showPartOfResult(URL url) {
      for (int i = 0; i < _fileList.size(); i++) {
         SearcherInfo si= (SearcherInfo)(_fileList.elementAt(i));
         try {
	     if (Utils.sameURL(url, new URL(si.getURLString()))) {
		 Vector v  = si.getFileList();
		 for (int j=0; j < v.size(); j++) {
		     ((SearcherWindow)_msw).addResultList(SearcherInfo.getFullPathName(url.toString(), (String)(v.elementAt(j))));
		 }
	     }
         } catch (Exception e) {
	     // not yet implemented
	 }
      }
   }


   /**
    * Dispatch a slave. 
    * @param destinations the destination URLs.
    * @param string the string to match against filenames. 
    */
    void go(Vector destinations, String str) { 
      super.go((URL)(destinations.firstElement()));
      try {
         Slave.create(null, SlaveClassName, getAgletContext(), 
                      this, new SeqItinerary(destinations), str);
      } catch (IOException ae) {
         inError(ae.getMessage());

      } catch (AgletException ae) {
         inError(ae.getMessage());
      }
   }

   //-- Write new values to the WhiteBoard.
   //
   void getNewFieldValues(Properties p) {
//       try {
	   for (Enumeration e=p.propertyNames(); e.hasMoreElements();) {
	       String key = (String)e.nextElement();
	       getAgletContext().setProperty(key,p.getProperty(key));
	   }
	   getAgletContext().multicastMessage(new Message("updateWindow"));
//       }  catch (AgletException ae) {
//	   inError(ae.getMessage());
//       }
   }
}
