/*
 * @(#)SearcherSlave.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */
 
package ibm.aglets.samples;

import aglet.*;
import ibm.aglets.patterns.*;

import java.util.Enumeration;
import java.util.Vector;
import java.util.StringTokenizer;

import java.net.URL;
import java.net.MalformedURLException;

import java.io.*;

/**
 * The SearcherSlave class is a aglet for the Searcher.
 *
 * @see Searcher
 * @version     1.00    96/12/28
 * @author      Danny B. Lange
 * @author      Yoshiaki Mima
 * @author      Yariv Aridor
 */

public class SearcherSlave extends Slave {
   private Vector _fileList = null;

   protected synchronized void initializeJob ()  {
      RESULT = new Vector();
   }

   protected synchronized void doJob() throws AgletException {
      String filename, readPath;

      filename = (String)ARGUMENT;
      _fileList = new Vector();

      //-- a slave is always untrusted
      AgletContext ctx = getAgletContext();
      readPath = (String)ctx.getProperty("aglets.tahiti.untrusted.read", "");
      if (!readPath.equals("")) {
         find(filename.toLowerCase(),    //-- case insensitive
              readPath.toLowerCase());
      } 

      ((Vector)RESULT).addElement(
         new SearcherInfo( getAgletContext().getHostingURL(), _fileList));
      reportMaster("Completed search");
   }
   

   //--  report the Master
   //
   private void reportMaster(String msg)  {
      AgletContext ac;    
      try {
         ac = getAgletContext();
         Messenger.create(ac, 
                          getOrigin(), 
			  getMaster(),
                          new Message ("status report",
                                       ac.getHostingURL().toString() + ":" + msg));
      } catch (IOException ae) {
        //-- we give up this report.
      } catch (AgletException ae) {
        //-- we give up this report.
      }
   }


   // looking for matched files (case insensitive)
   //  
   private void find(String s, String paths) {
      Enumeration e = makePathList(paths).elements();
      for (; e.hasMoreElements(); ) {
         File f = (File)e.nextElement();
         matchFiles(s, f);
      }
   }


   //-- match a filename with a given string. If the file 
   //-- is a directory, make recursive match.
   //
   private void matchFiles(String s, File f) {
      if (f.isFile()) {
         if (f.getAbsolutePath().toLowerCase().indexOf(s) > -1)   
            _fileList.addElement(f.getAbsolutePath());
      }
      else if (f.isDirectory() && f.canRead()) {
         // Get the directory list and perform a recursive descent.
         String[] dir = f.list();
         for (int i = 0; i < dir.length; i++) {
            matchFiles(s, new File(f, dir[i]));
         }
      }
   }

   //-- enumerate all allowed read paths..
   //
   private Vector makePathList(String p) {
      char ps = File.pathSeparatorChar;
      Vector pathList = new Vector();
      StringTokenizer st = new StringTokenizer(p, ps + ";" , false);

   	while (st.hasMoreTokens()) {
         pathList.addElement(new File(st.nextToken()));
/*
       // '/' is the universal path separator for path checking
	    path = path.replace(File.separatorChar, '/');
*/
      }
      return pathList;
   }



   //-- enumerate all allowed read paths..
   //
   private Vector makePathList1(String p) {
      // Separates each path in the path list and creates a vector of paths.
      char ps = File.pathSeparatorChar;
      Vector pathList = new Vector();
      int to = p.indexOf(ps);

      while (to < p.length()) {
         if (to == -1) {
            if (p.length() > 0) {
               String pp = p.substring(0).trim();
               if (!pp.equals(""))
                  pathList.addElement(new File(pp));
            }
            break;
         }
         String pp = p.substring(0, to).trim();
         if (!pp.equals(""))
            pathList.addElement(new File(pp));
         if (p.length() > to + 1) {
            p = p.substring(to + 1);
            to = p.indexOf(ps);
         } else
            break;
      }
      return pathList;
   }
}
