/*
 * @(#)SearcherWindow.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package ibm.aglets.samples;

import aglet.*;
import ibm.aglets.util.*;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Choice;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Label;
import java.awt.List;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.TextArea;
import java.awt.Color;
import java.awt.Insets;

import java.net.URL;
import java.net.MalformedURLException;

import java.util.Vector;
import java.util.Properties;
import java.util.Enumeration;
/**
 * Class SearcherWindow represents the main window for user interaction
 * with the Searcher.
 *
 * @see Searcher
 * @version  1.00    96/12/28
 * @author   Danny B. Lange
 * @author   Yoshiaki Mima
 * @author   Yariv Aridor
 */

public class SearcherWindow extends SampleWindow {
   private static final String TITLE = "Searcher";

   private static final String NO_DESTINATIONS_MSG = 
                  "Please insert at least one destination URL.";

   private PopUpMessageWindow _noDestinationsWindow = null ;


   //--  Constructs the dialog window.
   //
   public SearcherWindow(Searcher aglet) throws AgletException {
      super(aglet);
      _noDestinationsWindow = new PopUpMessageWindow(this, 
                                                     "DESTINATIONS ARE MISSING", 
                                                     NO_DESTINATIONS_MSG);
      makeMainPanel();
      displayFrame(this);
   }

   //-- Main panel
   private List _locationList   = new List(4, true);
   private List  _resultList    = new List(8, false);

   private void makeMainPanel() throws AgletException {
      Component comp;

      // button
      constraints.anchor = GridBagConstraints.CENTER;
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.BOTH;
      constraints.weightx = 1.0;
      comp = makeMainButtonPanel();
      layout.setConstraints(comp, constraints);
      add(comp);

      // title
      constraints.anchor = GridBagConstraints.WEST;
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.BOTH;
      constraints.weightx = 1.0;
      comp = new Label(TITLE);
      comp.setFont(new Font(getFont().getName(), 
                            Font.BOLD, 
                            getFont().getSize()+1));

      layout.setConstraints(comp, constraints);
      add(comp);

      // information settings
//      addLabeledComponent(URLLabel, _URLString);
//      initURLFields(getHotlist(_aglet));    // hotlist & URL field
//      addLabeledComponent(hotlistLabel, _hotlist);

      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.HORIZONTAL;
      constraints.insets = new Insets(0,0,10,0);
      constraints.weightx = 1.0;
      comp = _addressChooser;
      layout.setConstraints(comp, constraints);
      add(comp);

      // file name panel
      addLabeledComponent("File Name Substring :", _filepath);

      // panel for showing the visiting site
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.BOTH;
      comp = _locationList;
      layout.setConstraints(comp, constraints);
      add(comp);

      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.BOTH;
      constraints.weighty = 1.0;
      comp = makeLocalButtonPanel(_addLocation, _removeLocation, _showFiles);
      layout.setConstraints(comp, constraints);
      add(comp);

      // panel for showing the result
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.BOTH;
      constraints.weighty = 1.0;
      comp = _resultList;
      layout.setConstraints(comp, constraints);
      add(comp);

      // area for error messages
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.BOTH;
      constraints.weighty = 1.0;
      initMessagePanel();
      comp = _msgLine;
      layout.setConstraints(comp, constraints);
      add(comp);
   }


   //-- Result panel


   void addResultList(String s) {
      boolean found = false;
      int count = _resultList.countItems();
      String item; 

      for (int i = 0; i < count; i++) {
         item = _resultList.getItem(i);
         if (item.equals(s)) {
            return;
         }
      }

      _resultList.addItem(s);
      return;
   }

   public void clearResult() {
      _resultList.clear();
   }

   //--  Location panel
   private Button _addLocation = new Button("Add");
   private Button _removeLocation = new Button("Remove");
   private Button _showFiles = new Button("Show");
   
   private Panel makeLocalButtonPanel(Button b1, Button b2, Button b3) {
      Panel p = new Panel();
      p.setLayout(new FlowLayout(FlowLayout.RIGHT));
      p.add(b1);
      p.add(b2);
      p.add(b3);
      return p;
   }

   //--  The call back methods

   protected void go() {
      String filename = _filepath.getText().trim();
      String item;
      int size = _locationList.countItems();
      Vector itin; ;

      if (size == 0) {
         _noDestinationsWindow.popup(this);
         return;
      }   

      itin = new Vector();
      try {
         for (int i = 0; i < size; i++) {
            item = _locationList.getItem(i).trim();
            itin.addElement(new URL(item));
         }
         ((Searcher)_aglet).go(itin, filename);
      } catch (MalformedURLException e) {
         _malFormedURLWindow.popup(this);
      }
   }


   //--  add location to the list
   //
   private synchronized void addLocation() {
      int count;
      String newLocation;
      URL item, newURL;
      int port = -1;

      // new URL string
      newLocation = _addressChooser.getAddress();

      try {
          newURL = new URL(newLocation.toString());
      } catch (MalformedURLException e) {
         _malFormedURLWindow.popup(this);
         return;
      }

      count = _locationList.countItems();
      for (int i = 0; i < count; i++) {
         try  {
            item = new URL(_locationList.getItem(i));
         } catch (MalformedURLException e) {
            return;
         }

         if (Utils.sameURL(newURL,item)) {    //-- compare in the level of URLs.
            return;
         }
      }

      // if there is no matched URL, add a new item
      _locationList.addItem(newURL.toString().trim());
   }


   // remove selected item from the list
   //
   private void removeLocation() {
      for (int i = _locationList.countItems(); i >= 0; i--) {
         if (_locationList.isSelected(i)) {
            _locationList.delItem(i);
         }
      }
   }

   //-- show files found for the selected host in the list 
   // 
   private void showFiles() {
      URL url =  null;
      boolean found = false;
      for (int i = _locationList.countItems(); i >= 0; i--) {
         if (_locationList.isSelected(i)) {
            try {
               url = new URL(_locationList.getItem(i));
            } catch (MalformedURLException ae) {
               continue;
            }
            if (!found) {   //-- first time...
               found=true;
               clearResult();
            }
            ((Searcher)_aglet).showPartOfResult(url);
         }
      }
   }

//--  Event handler methods

   //--  The event handler.
   //
   public boolean handleEvent(Event event) {
     if (event.target instanceof List) 
        return handleList((List)event.target);
     else
        return super.handleEvent(event);
   }


   //--  Handles button events.
   //
   protected boolean handleButton(Button button) {
      if (button == _addLocation) {
         addLocation();
      } else if (button == _removeLocation) {
         removeLocation();
      } else if (button == _showFiles) {
         showFiles();
      } else 
	 return super.handleButton(button);
      return true;
   }


   protected boolean popUpHandleButton (Button button) {
      if (button == _malFormedURLWindow.getButton(PopUpMessageWindow.OKAY) &&
         "Okay".equals(button.getLabel()) ) {
         _malFormedURLWindow.hide();
         return true;
      }
      if (button == _noDestinationsWindow.getButton(PopUpMessageWindow.OKAY) &&
         "Okay".equals(button.getLabel()) ) {
         _noDestinationsWindow.hide();
         return true;
      }

      return false;  //-- should not reach here.
   }

   //--  Handles List field events.
   //
   private boolean handleList(List list) {
      if (list == _resultList) {
      	String str = list.getSelectedItem();
      	Properties p = new Properties();
	p.put("filename", SearcherInfo.extractPathName(str));
	p.put("location", SearcherInfo.extractHostURL(str));
	((Searcher)_aglet).getNewFieldValues(p);
      }
      return true;
   }
}


