/*
 * @(#)TransferWindow.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package ibm.aglets.samples;

import aglet.*;
import ibm.aglets.util.*;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Choice;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Label;
import java.awt.List;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.TextArea;
import java.awt.Color;
import java.awt.Insets;

import java.net.URL;
import java.net.MalformedURLException;

import java.util.Vector;

/**
 * Class TransferWindow represents the main window for user interaction
 * with the Transfer.
 *
 * @see Transfer
 * @version     1.00    96/12/28
 * @author      Danny B. Lange
 * @author      Yoshiaki Mima
 * @author      Yariv Aridor
 */

public class TransferWindow extends SampleWindow {
   private static final String TITLE = "Transfer";

   private PopUpMessageWindow _noFileNameWindow = null ;


   // Constructs the Filetransfer dialog window.
   //
   public TransferWindow(Transfer aglet) throws AgletException {
      super(aglet);
      _noFileNameWindow =
         new PopUpMessageWindow(this, "FILENAME NOT SPECIFIED ", NO_FILENAME_MSG);
      makeMainPanel();       
      displayFrame(this);
   }

   //-- Main panel


   private void makeMainPanel() throws AgletException {
      Component comp;
      
      // button
      constraints.anchor = GridBagConstraints.CENTER;
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.BOTH;
      constraints.weightx = 1.0;
      comp = makeMainButtonPanel();
      layout.setConstraints(comp, constraints);
      add(comp);

      // title
      constraints.anchor = GridBagConstraints.WEST;
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.BOTH;
      constraints.weightx = 1.0;
      comp = new Label(TITLE);
      comp.setFont(new Font(getFont().getName(), 
                            Font.BOLD, 
                            getFont().getSize()+1));

      layout.setConstraints(comp, constraints);
      add(comp);

      // information settings
//    addLabeledComponent(URLLabel, _URLString);
//    initURLFields(getHotlist(_aglet));    // hotlist & URL field
//    addLabeledComponent(hotlistLabel, _hotlist);

      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.HORIZONTAL;
      constraints.insets = new Insets(0,0,10,0);
      constraints.weightx = 1.0;
      comp = _addressChooser;
      layout.setConstraints(comp, constraints);
      add(comp);

      // panel of file name
      addLabeledComponent("File Name (Full Path):", _filepath);

      // panel for showing the result
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.BOTH;
      initResultPanel();
      comp = _result;
      layout.setConstraints(comp, constraints);
      add(comp);

      // area for error messages
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.BOTH;
      initMessagePanel();
      comp = _msgLine;
      layout.setConstraints(comp, constraints);
      add(comp);
   }

//--  The call back methods
//

   protected void go() {
      String fn = _filepath.getText().trim();
      String d = _addressChooser.getAddress();
      URL dest = null; 

      if (!fn.equals("")) {
         try { dest = new URL(d); 
         }  catch (MalformedURLException e) {
            _malFormedURLWindow.popup(this);
            return;
         } 
         ((Transfer)_aglet).go(dest, fn);
      } else {
         _noFileNameWindow.popup(this);
      }
   }


//--  Event handler methods


   protected boolean popUpHandleButton (Button button) {
      if (button == _malFormedURLWindow.getButton(PopUpMessageWindow.OKAY) &&
         "Okay".equals(button.getLabel()) ) {
          _malFormedURLWindow.hide();
          return true;
      }
      
      if (button == _noFileNameWindow.getButton(PopUpMessageWindow.OKAY) &&
         "Okay".equals(button.getLabel()) ) {
         _noFileNameWindow.hide();
         return true;
      }

      return false;  //-- should not reach here.
   }

}


