/*
 * @(#)Watcher.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package ibm.aglets.samples;

import aglet.*;
import ibm.aglets.patterns.*;
import ibm.aglets.util.*;

import java.net.URL;
import java.net.MalformedURLException;
import java.io.IOException;

import java.util.Vector;
import java.util.Hashtable;

/**
 * Class Watcher is used to check the update of a file located
 * in a remote aglet server. Given an URL and a file pathname, it will 
 * dispatch a slave (the WatcherNotifier class) to 'watch' update of a file.
 * The slave will not return to the master aglet. While at the remote server,
 * the slave may dispatch a file update report to the master. This is a part
 * of the Messenger usage pattern. Watcher is acting as a receiver in
 * this pattern.
 *
 * @see Watcher
 * @see WatcherWindow
 * @version     1.00     96/12/28
 * @author      Yoshiaki Mima
 * @author      Danny B. Lange
 * @author      Yariv Aridor
 */

public class Watcher extends SampleAglet  {
	
   private final static String NotifierClassName = 
                                 "ibm.aglets.samples.WatcherNotifier";


   public void onCreation (Object o) { 
      super.onCreation(o);
      try { _msw = new WatcherWindow(this);
            updateWindow();
      } catch (Exception e) {
          inError(e.getMessage()); 
      }
   }


   public boolean handleMessage(Message msg) {
     if ("notification".equals(msg.kind)) 
        message((Arguments)(msg.getArg()));
     else 
	super.handleMessage(msg);
     return true;
   }

   /**
    * Implements the message interface of a receiver. This method is
    * a part of the Messenger usage pattern.
    * @Messenger m Messanger aglet that has returened from a slave aglet.
    * @Object message Message returned.
    */
   private synchronized void message(Arguments message) {
     int type = ((Integer)message.getArgument("type")).intValue();
     if (type == Notifier.NOTIFICATION) {
        _msw.appendResult(
              "UPDATE: " + (String)(message.getArgument("message")) +
              ", AT: " + message.getArgument("date").toString());
     } else {
        setTheMessage((String)(message.getArgument("message")));
     }
   }


   /**
    * Creates and sets up the WatcherNotifier with the necessary information to
    * dispatch to a remote aglet server and hopefully stay their safely. This
    * method is a callback method for the interaction window.
    * @param itinerary contains the destination URL.
    */
   protected void go(URL destination, double interval, double duration, boolean stay, String path) {
      super.go(destination);
      try {
         Notifier.create(null, NotifierClassName, getAgletContext(), this,
            new SeqItinerary(destination), interval, duration, stay, path);
      } catch (IOException ae) {
         setTheMessage("Notifier Aglet creation failed...");
      } catch (AgletException ae) {
         setTheMessage("Notifier Aglet creation failed...");
      }
   }
}
