/*
 * @(#)Writer.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package ibm.aglets.samples;

import aglet.*;
import ibm.aglets.patterns.*;
import ibm.aglets.util.*;
import java.net.*;
import java.io.IOException;

/**
 * Class Writer is used to display a message on a remote aglet server. 
 * Given an URL (Agler Resource Locator) it will dispatch a slave (the 
 * WriterSlave class) to popup a window with the message on the remote
 * terminal. Finally, the slave returns to its origin host.
 *
 * @see WriterSlave
 * @version     1.00    96/12/28
 * @author      Yariv Aridor
 */

public class Writer extends SampleAglet {

   public void onCreation (Object o) { 
      super.onCreation(o);
      try { _msw = new WriterWindow(this);
            updateWindow();
      } catch (Exception e) {
          inError(e.getMessage()); 
      }
   }


   //-- Callback method for messages
   //
   protected synchronized void callback(Object obj) {
       if (obj != null) {
	   try { setTheMessage((String)obj);
	   } catch (Exception e) {
	       // not yet implemented
	   }
       }
   }


   //-- callback function for the "go" bottom.
   //
   protected void go(URL url, String text) {
      super.go(url);
      AgletContext ac = null;
      String username = "Unknown";
//      try {
            ac=getAgletContext();
	    username = (String)ac.getProperty("aglets.user.name", "Unknown");
//      } catch (AgletException ae) {
	// not yet handled.
//      }

      Arguments args = new Arguments();
      args.putArgument("msg",text);
      args.putArgument("user", username);
      try {
         Slave.create(null, "ibm.aglets.samples.WriterSlave", getAgletContext(),
                      this, new SeqItinerary(url), args);
      } catch (IOException ae) {
         inError(ae.getMessage());
      } catch (AgletException ae) {
         inError(ae.getMessage());
      }
   }   
}




