/*
 * @(#)WriterSlave.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package ibm.aglets.samples;

import aglet.*;
import ibm.aglets.patterns.*;
import ibm.aglets.util.*;
import java.util.Date;
import java.util.Vector;
import java.util.Enumeration;
import java.lang.InterruptedException;
import java.awt.Frame;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints ;
import java.awt.TextArea;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Choice;
import java.awt.Graphics;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Label;
import java.awt.List;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.Color;

import java.net.URL;
import java.net.MalformedURLException;

import java.util.Properties;


/**
 * WriterSlave is a aglet of Slave pattern that moves to a remote
 * site and displays a message
 *
 * @see Slave
 * @version     1.00    96/12/28
 * @author      Yariv Aridor
 *
 */

public class WriterSlave extends Slave {

    // time (in secs) of showing the message window.
   private final int SHOW_TIME = 10;


   protected  synchronized void initializeJob () {
      RESULT = null;
   }
   
   protected  synchronized void doJob() throws AgletException {
      WriterSlaveWindow win = null;
      try {
	  String from = new String ((String)(((Arguments)ARGUMENT).getArgument("user")) + 	                  
				    "@" +
				    getOrigin().getHost());
	  win = new WriterSlaveWindow(this,
				      (String)(((Arguments)ARGUMENT).getArgument("msg")),
				      from);
      }	catch (Exception e) {
	  e.printStackTrace();
	  throw new AgletException ("unable to create a remote message window");
      }
      try { wait(SHOW_TIME*1000);
      } catch (Exception e)  {
	  e.printStackTrace();
      }
      win.dispose();
      setResult("returned" + ((RESULT != null) ? ":" + (String)RESULT : "."));
  }


   void setResult(String text) {
       RESULT = text;
   }

   public synchronized void wakeup () {
       notify();
   }
}


class WriterSlaveWindow extends Frame {

   //-- size of message field
   private static final int SIZE = 40;
   //-- title of message window
   private static final String TITLE = "Writer";
   //-- thank message text
   private static final String THANKS = "THANKS!!";

   private GridBagLayout layout ;
   private TextField _messageText = new TextField(SIZE);
   private Aglet _aglet = null;

   //--  Constructs the dialog window.
   //
   public WriterSlaveWindow(Aglet ag, String text, String from) throws AgletException {
      super(TITLE);
      _aglet = ag;
      setLayout(layout = new GridBagLayout());
      Utils.setWindowProperties(this,_aglet);
      makeMainPanel(text, from);
      SampleWindow.displayFrame(this);
      writeMessage(text);
   }

   private void makeMainPanel(String text, String from) throws AgletException {
      Component comp;
      GridBagConstraints constraints = new GridBagConstraints();

      // button
      constraints.anchor = GridBagConstraints.CENTER;
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.BOTH;
      constraints.weightx = 1.0;
      comp = makeButtonPanel();
      layout.setConstraints(comp, constraints);
      add(comp);

      // title
      constraints.anchor = GridBagConstraints.WEST;
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.BOTH;
      constraints.weightx = 1.0;
      comp = new Label("MESSAGE FROM: " + from);
      layout.setConstraints(comp, constraints);
      add(comp);

      // message text panel
      constraints.gridwidth = GridBagConstraints.REMAINDER;
      constraints.fill = GridBagConstraints.BOTH;
      //_messageText.setEditable(false);
      comp = _messageText;
      layout.setConstraints(comp, constraints);
      add(comp);

   }

   protected Button _thanks = new Button("Thank");
   protected Button _quit = new Button("Quit");

   protected Panel makeButtonPanel() {
      Panel p = new Panel();
      p.setLayout(new FlowLayout(FlowLayout.RIGHT));
      p.add(_thanks);
      p.add(_quit);                                                         
      return p;
   }


   public void update (Graphics g) {
       paint(g);
   }


//-- Event Handler...

   public boolean handleEvent(Event event) {
      if (event.id == Event.ACTION_EVENT) {
         if (event.target instanceof Button) {
            return handleButton((Button)event.target);
         } 
      } else if (event.id == Event.WINDOW_ICONIFY) {
         hide();
         return true;
      } else if (event.id == Event.WINDOW_DESTROY) {
         quit();
         return true;
      }
      return super.handleEvent(event);
   }


   public void dispose() {
      super.dispose();
   }

   void quit() {
      dispose();
      ((WriterSlave)_aglet).wakeup();
   }

   void thanks () {
      ((WriterSlave)_aglet).setResult(THANKS);
      quit();
   }

   protected boolean handleButton(Button button) {
      if (button == _thanks) {
         thanks();
      } else if (button == _quit) {
         quit();
      }
      return true;
   }


  private void writeMessage (String text) {
     FontMetrics fm = getFontMetrics(getFont());
     int l = _messageText.size().width;
     int l3 = fm.charWidth(' ');
     try {
       for (int i=0; i<l; i++) { 
	  char msg[] = new char[(l-i)/l3]; 
	  for (int j=0 ; j<msg.length ; j++) {
	      msg[j] = ' ';
	  }
	  String str = new String(msg);
	  _messageText.setText(str + text);
	  // suspend to get the effect of text moving..
	  //((WriterSlave)_aglet).wait(200/1000);  
       }
     } catch (Exception e) {
	 e.printStackTrace();
     }
   }
}

