/*
 * @(#)ProxyWather.java
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package examples.watcher;

import aglet.*;

import java.net.URL;
import ibm.aglets.util.*;
import java.util.*;

/**
 * <tt> ProxyWatcher </tt> is an aglet which dispatches a slave aglet and
 * monitors proxies of the remote context. This is an example of
 * these features in Aglets.
 *
 * <ol>
 * <li> remote messaging
 * <li> delegation event model
 * <li> concurrency control by <tt> waitMessage()/notifyMessage() </tt>
 * <li> persistency
 * <li> activation by message
 * <li> remote control of an aglet.
 * </ol>
 * Please use JDK1.1 or later to compile these classes.
 * 
 * @version     1.00    $Date: 7/08/97 7:11p $
 * @author      Mitsuru Oshima
 * @see examples.watcher.WatcherSlave
 */
public class ProxyWatcher extends Aglet {
    transient AgletContext ac;
    WatcherFrame frame;
    AgletProxy slave = null;

    public void onCreation(Object o) {
	ac = getAgletContext();
	frame = new WatcherFrame(this);
	frame.pack();
	frame.show();
    }

    public boolean handleMessage(Message msg) {
	if (msg.sameKind("update")) {
	    String s = String.valueOf(msg.getArg());
	    frame.update(s);
	} else return false;
	return true;
    }

    public void go(String address) {
	try {
	    //
	    // Creates another aglet
	    //
	    slave = ac.createAglet(getCodeBase(),
				   "examples.watcher.WatcherSlave",
				   getAgletID());
	    //
	    // Obtain the remote proxy.
	    //
	    slave = slave.dispatch(new URL(address));
	} catch (Exception ex) {
	    if (slave != null) {
		slave.dispose();
	    }
	    ex.printStackTrace();
	}
    }

    public void sendMessage(Message msg) {
	if (slave != null) {
	    try {
		//
		// Remote messaging.
		//
		slave.sendAsyncMessage(msg);
	    } catch (InvalidAgletException ex) {
		ex.printStackTrace();
	    }
	}
    }

    public void move(String address) {
	try {
	    URL dest = new URL(address);
	    slave = slave.dispatch(dest);
	} catch (AgletException ex) {
	    ex.printStackTrace();
	} catch (java.io.IOException ex) {
	    ex.printStackTrace();
	}
    }

    public void terminate() {
	if (slave == null) {
	    return;
	}
	try {
	    //
	    // you can call the dispose method on the remote aglet.
	    //
	    slave.dispose();
	} catch (InvalidAgletException ex) {
	    ex.printStackTrace();
	}
    }
}

class WatcherFrame extends java.awt.Frame {
    ProxyWatcher aglet;
    java.awt.TextArea text = new java.awt.TextArea(10,10);
    ibm.aglets.util.AddressChooser address =
	    new ibm.aglets.util.AddressChooser(15);
    
    WatcherFrame(ProxyWatcher a) {
	aglet = a;
	setLayout(new java.awt.BorderLayout());
	add("North", address);
	add("Center", text);
	java.awt.Panel p = new java.awt.Panel();
	p.setLayout(new java.awt.FlowLayout());

	p.add(new java.awt.Button("Go!"));
	p.add(new java.awt.Button("Start"));
	p.add(new java.awt.Button("Stop"));
	p.add(new java.awt.Button("Sleep"));
	p.add(new java.awt.Button("Move"));
	p.add(new java.awt.Button("Terminate"));
	add("South", p);
    }

    public boolean handleEvent(java.awt.Event ev) {
	if (ev.id == java.awt.Event.WINDOW_DESTROY) {
	    dispose();
	    return true;
	}
	return super.handleEvent(ev);
    }

    public boolean action(java.awt.Event ev, Object obj) {
	if (ev.target instanceof java.awt.Button) {
	    java.awt.Button b = (java.awt.Button)ev.target;
	    String l = b.getLabel();
	    if ("Go!".equals(l)) {
		aglet.go(address.getAddress());
	    } else if ("Start".equals(l)) {
		aglet.sendMessage(new Message("start"));
	    } else if ("Stop".equals(l)) {
		aglet.sendMessage(new Message("stop"));
	    } else if ("Sleep".equals(l)) {
		aglet.sendMessage(new Message("sleep"));
	    } else if ("Move".equals(l)) {
		aglet.move(address.getAddress());
	    } else if ("Terminate".equals(l)){
		aglet.terminate();
	    }
	} else return false;
	return true;
    }

    void update(String s) {
	text.setText(s);
    }
}
