/*
 * @(#)AgletInfo.java
 *
 * (c) Copyright IBM Corp. 1997
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package aglet;

import java.net.URL;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;

/**
 * AgletInfo class is a object which contains the information of the aglet.
 *
 * @version     1.00	$Date: 7/07/97 6:07p $
 * @author	Mitsuru Oshima
 */
final public class AgletInfo implements java.io.Externalizable {

    /**
     * Constructs a AgletInfo with the specified parameters.
     * This will be removed in the future release.
     * @param aid     the aglet identifier
     * @param address the address where the aglet currently resides
     * @param classname the classname of the aglet
     * @param codebase the codebase of the aglet classes
     * @param origin   the address where the aglet was instantiated
     * @param birthtime the time when the aglet was instantiated
     * @param api_major_version
     * @param api_minor_version
     * @param authorityName the authority's name of the aglet
     * @param privilegeName the name of the privilege in the current
     *                      environment.
     * @deprecated
     */
    public AgletInfo(AgletID aid,
		     String address,
		     String classname,
		     URL codebase,
		     String origin,
		     long birthtime,
		     short api_major_version,
		     short api_minor_version,
		     String authorityName,
		     String privilegeName) {
	this.aid = aid;
	this.address   = address;
	this.classname = classname;
	this.codebase  = codebase;
	this.origin    = origin;
	this.birthtime = birthtime;
	this.api_major_version = api_major_version;
	this.api_minor_version = api_minor_version;
	this.authorityName = authorityName;
	this.privilegeName = privilegeName;
    }

    /**
     * Constructs a AgletInfo with no parameter.
     * @deprecated
     */
    public AgletInfo() {
    }
     
    /**
     * Gets the aglet id
     */
    public AgletID getAgletID() {
	return aid;
    }

    /**
     * Gets the class name of the aglet
     */
    public String getAgletClassName() {
	return classname;
    }

    /**
     * Gets the origin address where the aglet was instantiated
     */
    public String getOrigin() {
	return origin;
    }

    /**
     * Gets the codebase address where the aglet code resides
     */
    public URL getCodeBase() {
	return codebase;
    }

    /**
     * Gets the address where the aglet currently resides
     */
    public String getAddress() {
	return address;
    }

    /**
     * Gets the time when the aglet is created
     */
    public long getCreationTime() {
	return birthtime;
    }

    /**
     * Gets the API major version to which the aglet is instantiated
     */
    public short getAPIMajorVersion() {
	return api_major_version;
    }

    /**
     * Gets the API minor version to which the aglet is instantiated
     */
    public short getAPIMinorVersion() {
	return api_minor_version;
    }

    /**
     * Gets the authority name. This will be replaced with new Security API.
     * Please do not use this method for the moment.
     * @deprecated
     */
    public String getAuthorityName() {
	return authorityName;
    }

    /**
     * Gets the privilege name.  This will be replaced with new Security API.
     * Please do not use this method for the moment.
     * @deprecated
     */
    public String getPrivilegeName() {
	return privilegeName;
    }

    /**
     * Write the aglet info to the data output object. It is not normally
     * used by the aglet programmers. This will be removed in the future
     * release.
     * @deprecated
     */
    public void writeTo(DataOutput s, URL cb) throws IOException {
	byte b[] = aid.toByteArray();
	s.writeInt(b.length);
	s.write(b);

	s.writeUTF(address);
	s.writeUTF(classname); 
	s.writeUTF(cb.toString());// for JDK1.0.2
	s.writeUTF(origin);
	s.writeLong(birthtime);
	s.writeShort(api_major_version);
	s.writeShort(api_minor_version);

	s.writeUTF(authorityName);
	s.writeUTF(privilegeName);
    }

    /**
     * Reads the aglet info from DataInput object. It is not normally used by
     * the aglet programmers. This will be removed in the future release.
     * @deprecated
     */
    static public AgletInfo readFrom(DataInput s) throws IOException {
	AgletInfo info = new AgletInfo();
	info.read(s);
	return info;
    }

    /**
     * @deprecated
     */
    public void writeExternal(java.io.ObjectOutput out) throws IOException {
	writeTo(out, codebase);
    }

    /**
     * @deprecated
     */
    public void readExternal(java.io.ObjectInput out) throws IOException, ClassNotFoundException {
	read(out);
    }

    private void read(DataInput in) throws IOException {
	int len = in.readInt();
	byte b[] = new byte[len];
	in.readFully(b);

	aid = new AgletID(b);
	address   = in.readUTF();
	classname = in.readUTF();
	codebase  = new URL(in.readUTF());	// for JDK1.0.2
	origin    = in.readUTF();
	birthtime = in.readLong();
	api_major_version = in.readShort();
	api_minor_version = in.readShort();

	authorityName = in.readUTF();
	privilegeName = in.readUTF();
    }

    /**
     * Converts to a string
     */
    public String toString() {
	StringBuffer b = new StringBuffer();
	b.append("ID       : " + aid + '\n');
	b.append("Address  : " + address + '\n');
	b.append("ClassName: " + classname + '\n');
	b.append("Origin   : " + origin + '\n');
	b.append("CodeBase : " + codebase + '\n');
	return b.toString();
    }

    private AgletID aid;
    private String address;
    private String classname;
    private String origin;
    private URL codebase;
    private long birthtime;
    private short api_major_version;
    private short api_minor_version;

    //
    private String authorityName = null;
    private String privilegeName = null;
}
