/*
 * @(#)AgletStub.java
 *
 * (c) Copyright IBM Corp. 1997
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package aglet;

import aglet.event.*;
import java.net.URL;
import java.io.IOException;

/**
 * Abstract class AgletStub is used to implement an aglet behavior.
 * It is not normally used by aglet programmers.
 *
 * @version     1.10    97/06/13
 * @author	Mitsuru Oshima
 */
abstract public class AgletStub {

    /*
     * Constructs AgletProxy
     */
    protected AgletStub() {
    }

    /*
     * Sets a aglet.
     */
    abstract protected void setAglet(Aglet aglet);

    /**
     * Gets the current content of the Aglet's message line.
     * @return the message line.
     * @exception InvalidAgletException if the aglet is not valid.
     */
    abstract protected String getText();

    /*
     * Gets the aglet context in which the aglet lives.
     * @exception InvalidAgletException if the aglet is invalid.
     */
    abstract protected AgletContext getAgletContext();

    /**
     * Gets the aglet's message manager object.
     * @see aglet.MessageManager
     * @return the method manager
     * @exception InvalidAgletException if the aglet is not valid.
     */
    abstract protected MessageManager getMessageManager();

    /*
     * Gets the info
     */
    abstract protected AgletInfo getAgletInfo();

    /**
     * Sets a aglet's text
     */
    abstract protected void setText(String text);

    /*
     * subscribe the specific multicast message
     */
    abstract protected void subscribeMessage(String name);
   
    /*
     * unsubscribe the message
     */
    abstract protected boolean unsubscribeMessage(String name);

    /*
     * unsubscribe All messages the aglet had been subscribing.
     */
    abstract protected void unsubscribeAllMessages();

    /*
     * Clones the aglet
     */
    abstract protected Object clone() throws CloneNotSupportedException ;

    /*
     * Disposes the aglet
     */
    abstract protected void dispose();

    /*
     * Dispatches the aglet
     */
    abstract protected void dispatch(URL url) throws IOException, RequestRefusedException;

    /*
     * Deactivates the aglet
     */
    abstract protected void deactivate(long duration) throws IOException;

    /*
     * Take a snapshot of the aglet into the 2nd strage. This will be
     * activated only if the system clashed without being dispatched,
     * deactivated or disposed.
     */
    abstract protected void snapshot() throws IOException ;

    /*
     * Add a clone event listner
     */
    abstract protected void addCloneListener(CloneListener l);

    /*
     * Removes a clone event listner
     */
    abstract protected void removeCloneListener(CloneListener l);

    /*
     * Add a mobility event listner
     */
    abstract protected void addMobilityListener(MobilityListener l);

    /*
     * Removes a mobility event listner
     */
    abstract protected void removeMobilityListener(MobilityListener l);

    /*
     * Add a persistency event listner
     */
    abstract protected void addPersistencyListener(PersistencyListener l);

    /*
     * Removes a persistency event listner
     */
    abstract protected void removePersistencyListener(PersistencyListener l);
}
