/*
 * @(#)Itinerary.java
 *
 * (c) Copyright IBM Corp. 1996
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package aglet.a4compatibility;

import aglet.AgletProxy;
import aglet.AgletException;
import aglet.InvalidAgletException;

import java.util.Enumeration;

/**
 * The <tt>Itinerary</tt> is an abstract base class for control over
 * an aglet's tour. Use this class to create specialized Itinerary
 * classes. An Itinerary object is bound to an Aglet by the Aglet's <tt>
 * setItinerary</tt> method. Any Itinerary object can be bound to at most
 * one specific aglet
 * (known as the traveller).
 *
 * @version     1.00  96/09/04                                         
 * @author      Danny B. Lange                                           
 * @author      Yariv Aridor
 * @author	 Mitsuru Oshima
 */ 
  
abstract public class Itinerary {

    /*
     * the traveller
     */
    transient protected AgletProxy traveller = null;

    /*
     * Sets an aglet
     */
    final public void setTraveller(AgletProxy proxy) {
	if (traveller != null) {
	    throw new SecurityException();
	}
	traveller = proxy;
    }

    /*
     */
    final protected void unsetTraveller() {
	traveller = null;
    }

    /*
     *
     */
    protected void initialize() {
    }

    /*
     *
     */
    public void onArrival() {
    }

    public static final String FIRST = "_first";
    public static final String LAST  = "_last";
    public static final String NEXT  = "_text";
    public static final String PREVIOUS = "_previous";
    /*
     * symbolic name can be any symbolic name which the itinerary
     * can understands. It could be
     * '_first'
     * 'secretary'
     * 'error_handler'
     * 'moshima@trl.ibm.co.jp'
     * 'atp://java.trl.ibm.com:434'
     * and so forth.
     */
    abstract public void go(String name) throws java.io.IOException, AgletException;

    public void go(String name, Object arg) throws java.io.IOException, AgletException {
    }

    abstract public boolean isCurrent(String symbolic_name) throws InvalidAgletException;

    abstract public boolean isAvailable(String symbolic_name) throws InvalidAgletException;

    abstract public void reset() throws InvalidAgletException;

    abstract public String[] keywords();
}

