/*
 * @(#)Aglets.java
 *
 * (c) Copyright IBM Corp. 1997
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package aglet.system;

import aglet.*;
import java.net.URL;
import ibm.awb.util.Resource;

/**
 * Aglets class defines a set of convenient functions for a client which
 * has no AgletContext and daemon to receive incoming aglets.
 *
 * @version     1.00    $Date: 7/08/97 7:11p $
 * @author      Mitsuru Oshima
 */
public class Aglets {

    static {
	try {
	    Class.forName("java.net.URL");
	    String handler_pkgs =
		    System.getProperty("java.protocol.handler.pkgs", "");

	    if (handler_pkgs.indexOf("ibm.net.protocol") < 0) {
		java.util.Properties props = System.getProperties();

		handler_pkgs += "|sun.www.net.protocol|ibm.net.protocol";
		props.put("java.protocol.handler.pkgs", handler_pkgs);
	    }
	} catch (SecurityException ex) {
	    ex.printStackTrace();
	} catch (Throwable ex) {
	    ex.printStackTrace();
	}

	try {
	    if (Resource.getResourceFor("atp") == null) {
		// create default resource
		Resource.createResource("atp", null, null);
	    }
	    if (Resource.getResourceFor("aglets") == null) {
		Resource.createResource("aglets", null, null);
	    }
	} catch (Throwable t) {
	    t.printStackTrace();
	}
    }

    /*
    public static boolean isGuiAvailable() {
	return AgletRuntime.getAgletRuntime().isGuiAvailable();
    }
    */

    /**
     * Creates an aglet
     *
     */
    public static AgletProxy createAglet(String contextAddress,
					 URL codebase,
					 String classname,
					 Object init) {
	return AgletRuntime.getAgletRuntime().createAglet(contextAddress,
							  codebase,
							  classname,
							  init);
    }

    /**
     * Obtains a proxy reference the remote aglet.
     */
    public static AgletProxy getAgletProxy(String contextAddress, AgletID id) {
	return AgletRuntime.getAgletRuntime().getAgletProxy(contextAddress, id);
    }

    /**
     * Gets an enumeration of aglet proxies of all aglets residing
     * in the context specified by contextAddress.
     * @param contextAddress
     */
    public static AgletProxy[] getAgletProxies(String contextAddress) {
	return AgletRuntime.getAgletRuntime().getAgletProxies(contextAddress);
    }

    /**
     *
    public static Enumeration getAgletProxies(String contextAddress, String AuthorityName) {
	return AgletRuntime.getAgletRuntime().getAgletProxies(contextAddress, AuthorityName);
    }
     */
}
