/*
 * @(#)AgentRequestHandler.java
 *
 * (c) Copyright IBM Corp. 1996
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package atci;

import java.io.IOException;
import java.net.URL;

/**
 * An abstract class to handle incoming requests
 *
 * @version     1.11	24 Mar 1997
 * @author	Mitsuru Oshima
 */
public abstract class AgentRequestHandler {

    abstract public void initialize(URL hosting);

    /**
     * Dispatches requests to specific handle methods. You are free to
     * override this method to handle them by your self.
     *
     * @see atci.AtciResponse
     * @see atci.AtciResponse
     */
    public void handleRequest(AtciRequest request, AtciResponse response) throws IOException {
	switch( request.getMethod() ) {
	case AtciConstants.DISPATCH:
	    handleDispatchRequest(request, response);
	    break;

	case AtciConstants.RETRACT:
	    handleRetractRequest(request, response);
	    break;

	case AtciConstants.FETCH:
	    handleFetchRequest(request, response);
	    break;

	    /*
	case AtciConstants.PING:
	    handlePingRequest(request, response);
	    break;
	    */

	case AtciConstants.MESSAGE:
	    handleMessageRequest(request, response);
	    break;
	default:
	    response.sendError(AtciConstants.BAD_REQUEST);
	    break;
	}
    }

    /**
     *
     */
    protected void handleDispatchRequest(AtciRequest request, AtciResponse response) throws IOException {
	response.sendError(AtciConstants.NOT_IMPLEMENTED);
    }

    /**
     *
     */
    protected void handleRetractRequest(AtciRequest request, AtciResponse response) throws IOException {
	response.sendError(AtciConstants.NOT_IMPLEMENTED);
    }

    /**
     *
     */
    protected void handleFetchRequest(AtciRequest request, AtciResponse response) throws IOException {
	response.sendError(AtciConstants.NOT_IMPLEMENTED);
    }
	    
    /**
     *
    protected void handlePingRequest(AtciRequest request, AtciResponse response) throws IOException {
	response.sendError(AtciConstants.NOT_IMPLEMENTED);
    }
     */

    /**
     *
     */
    protected void handleMessageRequest(AtciRequest request, AtciResponse response) throws IOException {
	response.sendError(AtciConstants.NOT_IMPLEMENTED);
    }
}
