/*
 * @(#)AtciConnector.java
 *
 * (c) Copyright IBM Corp. 1996
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package atci;

import java.net.URL;
import java.net.MalformedURLException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Properties;

/**
 * @version     1.10	23 Mar 1997
 * @author	Mitsuru Oshima
 */
public class AtciConnector {

    private static AtciConnector currentConnector = null;
    static private Hashtable handlers = new Hashtable();
    static public String protocolPathProp = "atci.protocol.handler.pkgs";

    // temoprary (used by Fiji)

    static private String handlerPkgs = null;

    /**
     * Defines the ATCI handler packages
     * @param packages the packages
     */
    static public void initialize(String pkgs) {
//	if (false) { handlerPkgs !=null) {
//	    throw new SecurityException ("handler packages already set");
//	}
//	else {
	    handlerPkgs = pkgs;
	    String pkg="ibm.atp.protocol";
	    handlerPkgs = "|" + pkg;
//	}
    }


    /**
     * Sets a connector
     *
     * @param connector
     */
    synchronized static public void setAtciConnector(AtciConnector connector) {
	if (currentConnector != null) {
	    throw new SecurityException("connector already set");
	}
	currentConnector = connector;
    }

    /**
     * Gets the connector
     */
    synchronized static public AtciConnector getAtciConnector() {
	if (currentConnector == null) {
	    currentConnector = new AtciConnector();
	}
	return currentConnector;
    }

    /**
     * Opens connection
     *
     * @see openConnectionFor
     * @param url
     */
    static public AtciConnection openConnection(URL url) throws IOException, MalformedURLException {
	return getAtciConnector().openConnectionFor(url);
    }

    public AtciConnector() {
    }

    /**
     * Opens connection
     *
     * @param url
     */
    public AtciConnection openConnectionFor(URL url) throws IOException, MalformedURLException {
	String protocol = url.getProtocol();
	AtciConnectionHandler handler = (AtciConnectionHandler)handlers.get(protocol);
	if (handler == null) {
	    handler = getAtciConnectionHandler(protocol);
	    if (handler != null) {
		handlers.put(protocol, handler);
	    }
	    if (handler == null) {
		throw new MalformedURLException("unknown protocol: " + protocol);
	    }
	}
	return handler.openConnection(url);
    }

    /**
     * Gets a AtciConnection handler for given sheme(protocol).
     *
     * @param prototocol scheme used in URL object to connect
     */
    protected AtciConnectionHandler getAtciConnectionHandler(String protocol) {
	AtciConnectionHandler handler =
		(AtciConnectionHandler)handlers.get(protocol);
	if (handler == null) {

	    String packagePrefixList = "";
	    // temporary:
	    try { 
		packagePrefixList = System.getProperty(protocolPathProp,"");
		if (packagePrefixList != "") {
		    packagePrefixList += "|";
		}
		packagePrefixList += "ibm.atp.protocol";
	    } catch (SecurityException e) {  // fiji will throw that exception
		packagePrefixList=handlerPkgs;
	    }

	    StringTokenizer packagePrefixIter =
		    new StringTokenizer(packagePrefixList, "|");

	    while (handler == null && packagePrefixIter.hasMoreTokens()) {
		String packagePrefix = packagePrefixIter.nextToken().trim();
		try {
		    String clname = packagePrefix + "." + protocol
			    + ".Handler";
		    handler = (AtciConnectionHandler)Class.forName(clname).newInstance();
		} catch (Exception e) {
		}
	    }
	}
	if (handler != null) {
	    handlers.put(protocol, handler);
	}
	return handler;
    }

}

