/*
 * @(#)CommandLine.java
 *
 * (c) Copyright IBM Corp. 1996
 *
 * IBM grants you a non-exclusive, non-transferrable License to
 * use this program internally solely for the purposes of testing
 * and evaluating Java Aglet API.
 * You may not distribute, sublicense, lease, rent or use this
 * sample program externally.
 *
 * THIS ROGRAM IS PROVIDED "AS IS" WITHOUT ANY WARRANTY EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE WARRANTY OF
 * NON-INFRINGEMENT AND THE WARRANTIES OF MERCHANTIBILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE.
 * IBM WILL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY YOU AS
 * A RESULT OF USING THIS SAMPLE PROGRAM. IN NO EVENT WILL IBM BE
 * LIABLE FOR ANY SPECIAL, INDIRECT CONSEQUENTIAL DAMAGES OR LOST
 * PROFITS EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF THEIR
 * OCCURRENCE OR LOSS OF OR DAMAGE TO YOUR RECORDS OR DATA.
 * IBM WILL NOT BE LIABLE FOR ANY THIRD PARTY CLAIMS AGAINST YOU.
 */

package ibm.aglets.tahiti;

import aglet.*;
import aglet.system.ContextEvent;
import aglet.system.ContextListener;
import aglet.system.AgletRuntime;

//
import ibm.awb.util.Resource;

import java.io.DataInputStream;
import java.io.DataInput;
import java.io.IOException;
import java.io.PrintStream;

import java.net.URL;

import java.util.StringTokenizer;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Date;

/**
 * <tt>CommandLine</tt> is a simple command line interface to an aglet server.
 *
 * @see ibm.aglets.AgletsViewer
 * @see ibm.aglets.AgletEvent
 * @version     1.10    96/10/15
 * @author	Mitsuru Oshima
 */
public final class CommandLine implements ContextListener, Runnable {

    private static String helpMsg = 
	    "help                    Display this message. \n" +
	    "shutdown                Shutdown the server. \n" +
	    "reboot                  Reboot the server. \n" +
	    "list                    List all aglets in the server. \n" +
	    "prompt                  Display or changes the prompt. \n" +
	    "create [codeBase] name  Create new aglet. \n" +
	    "<aglet> dispatch URL    Dispatch the aglet to the URL. \n" +
            "<aglet> clone           Clone the aglet. \n" +
	    "<aglet> dispose         Dispose the aglet. \n" +
            "<aglet> dialog          Request a dialog to interact with.\n" +
            "<aglet> property        Display properties of the aglet.\n" +
	    "Note: <aglet> is a left most string listed in the result of list command. ";

    /*
     * for autonumbering aglets
     */
    private static int serial = 0;

    /*
     * Aglets table
     */
    private Hashtable aglets = new Hashtable();

    /*
     * Prompt char.
     */
    private String prompt = ">";

    /*
     * AgletContext
     */
    private AgletContext context = null;

    /*
     * Null print stream to turn off messages.
    private ibm.awb.util.LogPrintStream null_log;
     */
    private boolean message = true;

    /*
     *
     */
    private void inputResource(String title, String res_name) {
	String line = null;
	DataInput in = new DataInputStream(System.in);
	Resource res = Resource.getResourceFor("aglets");
	String defval = res.getString(res_name, null);

	while(true) {
	    System.out.print(title + 
			     (defval == null ? ":" :
			      "[" + defval + "]:"));
	    System.out.flush();
	    try {
		line = in.readLine();
	    } catch (IOException ex) {
	    }
	    if (line == null) {
		System.exit(1);
	    }
	    if (line.trim().length() != 0) {
		
		res.setResource(res_name, line);
		return;
	    } else if (defval != null && defval.trim().length() != 0) {
		return;
	    }
	}
    }

    /**
     * Initializes the Viewer with the context.
     * @param context the AgletContext assigned to this Viewer.
     */
    public void contextStarted(ContextEvent ev) {
	this.context = ev.getAgletContext();
	Resource res = Resource.getResourceFor("aglets");

	//
	// Check to see if this is a registered user.
	//
	if ("".equals(res.getString("aglets.user.organization","")) ||
	    "".equals(res.getString("aglets.user.email","")) ||
	    res.getBoolean("aglets.registered",false) == false) {

	    String disclaimer =
		" ============= Registration ============ \n"+
		"Sending an aglet is like sending an email. Please provide the\n" +
		"the system with the requested personal information. This information\n" +
		"will be incorporated into aglets created by you and serve to identify\n" +
		"the creator/sender of an aglet.";

	    System.out.println(disclaimer);

	    while(true) {
		inputResource("Full Name", "aglets.user.name");
		inputResource("Organization", "aglets.user.organization");
		inputResource("E-mail Address", "aglets.user.email");

		System.out.println("OK?(y/n)");
		try {
		    if (System.in.read() == 'y') {
			res.setResource("aglets.registered","true");
			res.save("Registration");
			break;
		    }
		} catch (IOException ex) {
		    ex.printStackTrace();
		}
	    }
	}

	// initialize gui....
	try {
	    Class.forName("sun.awt.image.JPEGImageDecoder");
	    Class.forName("sun.audio.AudioPlayer");
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
	try {
	    Class.forName("sun.awt.PlatformFont"); // for 1.1
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
	try {
	    new java.awt.Frame().addNotify();
	} catch (Exception ex) {
	    ex.printStackTrace();
	} finally {
	}

	try {
	    Class.forName("sun.awt.ScreenUpdater");
	} catch (Throwable t) {
	}

	new Thread(this).start();
    }

    public void contextShutdown(ContextEvent ev) {
	//
    }

    /**
     * Infinite loop to process inputs
     */
    public void run() {
	Thread.currentThread().setPriority(1);
	DataInput in = new DataInputStream(System.in);
	while(true) {
	    try {
		System.out.print( prompt + " ");
		System.out.flush();
		String line = in.readLine();
		command(line);
	    } catch (Throwable ex) {
		ex.printStackTrace();
	    }
	}
    }

    /**
     * Interprets inputs from command line.
     *
     * @param line typed string
     */
    void command(String line) throws Exception {
	Item item = null;
	StringTokenizer st = new StringTokenizer(line, " \t");

	if (st.hasMoreTokens()) {
	    String cmd = st.nextToken();

	    if ("shutdown".equalsIgnoreCase(cmd)) {
		AgletRuntime.getAgletRuntime().shutdown();
//		AgletsSystem.shutdown();
		System.exit(1);
	    } else if ("reboot".equalsIgnoreCase(cmd)) {
		AgletRuntime.getAgletRuntime().shutdown();
//		AgletsSystem.shutdown();
		System.exit(0);
	    } else if ("list".equalsIgnoreCase(cmd)) {
		list(System.out);
	    } else if ("threads".equalsIgnoreCase(cmd)) {
		MainWindow.showThreads();
	    } else if ("debug".equalsIgnoreCase(cmd)) {
		if (st.hasMoreTokens()) {
		    if ("on".equalsIgnoreCase(st.nextToken())) {
			ibm.awb.util.Debug.debug(true);
		    } else {
			ibm.awb.util.Debug.debug(false);
		    }
		} else {
		    ibm.awb.util.Debug.list(System.err);
		}
	    } else if ("msg".equalsIgnoreCase(cmd)) {
		if (st.hasMoreTokens()) {
		    if ("on".equalsIgnoreCase(st.nextToken())) {
			message = true;
		    } else {
			message = false;
		    }
		} else {
		    System.out.println(message ? "message on":"message off" );
		}
	    } else if ("prompt".equalsIgnoreCase(cmd)) {
		if (st.hasMoreTokens()) {
		    prompt = st.nextToken();
		} else {
		    System.out.println("prompt : " + prompt);
		}
	    } else if ("help".equalsIgnoreCase(cmd)) {
		System.out.println(helpMsg);
	    } else if ("create".equalsIgnoreCase(cmd)) {
		URL url = null;
		String name = "";
		if (st.countTokens()==2) {
		    url = new URL(st.nextToken());
		    name = st.nextToken();
		} else if (st.countTokens()==1) {
		    name = st.nextToken();
		} else {
		    System.out.println("create [URL] name");
		    return;
		}
		context.createAglet(url, name, null);
	    } else if ( (item=(Item)aglets.get(cmd)) != null) {
		if (st.hasMoreTokens()) {
		    item.command( st.nextToken(), st);
		} else {
		    System.out.println(item);
		}
	    } else {
		System.out.println("unknown command : " + cmd);
	    }
	}
    }

    synchronized private void removed(AgletProxy proxy) {
	Enumeration e = aglets.keys();
	while(e.hasMoreElements()) {
	    Object k = e.nextElement();
	    if ( aglets.get(k).equals(proxy) ) {
		aglets.remove(k);
	    }
	}
    }

    synchronized private void added(AgletProxy proxy) {
	aglets.put("aglet" + (serial++), new Item(proxy));
    }

    /**
     * Called when an aglet has been created
     * @param event an AgletEvent
     */
    public void agletCreated(ContextEvent event) {
	added(event.getAgletProxy());
    }

    /**
     * Called when an aglet has been cloned
     * @param event an ContextEvent
     */
    public void agletCloned(ContextEvent event) {
	added(event.getAgletProxy());
    }

    /**
     * Called when an aglet has arived
     * @param event an ContextEvent
     */
    public void agletArrived(ContextEvent event) {
	added(event.getAgletProxy());
    }

    /**
     * Called when an aglet has been activated
     * @param event an ContextEvent
     */
    public void agletActivated(ContextEvent event) {
	added(event.getAgletProxy());
    }

    /**
     * Called when an aglet has been disposed
     * @param event an ContextEvent
     */
    public void agletDisposed(ContextEvent event) {
	removed(event.getAgletProxy());
    }

    /**
     * Called when an aglet has been dispatched
     * @param event an ContextEvent
     */
    public void agletDispatched(ContextEvent event) {
	removed(event.getAgletProxy());
    }

    /**
     * Called when an aglet has been reverted
     * @param event an ContextEvent
     */
    public void agletReverted(ContextEvent event) {
	removed(event.getAgletProxy());
    }

    /**
     * Called when an aglet has been deactivated
     * @param event an ContextEvent
     */
    public void agletDeactivated(ContextEvent event) {
	removed(event.getAgletProxy());
    }

    /**
     * Called when an aglet has been updated
     * @param event an ContextEvent
     */
    public void agletStateChanged(ContextEvent event) {
	synchronized(aglets) {
	    Enumeration e = aglets.keys();
	    while(e.hasMoreElements()) {
		Object k = e.nextElement();
		Item i = (Item)aglets.get(k);
		if (i.equals(event.getAgletProxy())) {
		    i.setText((String)event.arg);
		    System.out.println(k.toString() + " : " + event.arg);
		}
	    }
	}
    }

    /**
     * Called to show the message
     * @param event an ContextEvent
     */
    public void showMessage(ContextEvent event) {
	if (message) {
	    System.out.println((String)event.arg);
	}
    }

    /**
     * Called when an aglet request to show the document given
     * as URL
     * @see Aglet#showDocument
     * @param event an ContextEvent
     */
    public void showDocument(ContextEvent event) {
	System.out.println("hyper link required :" + event.arg);
    }

    /**
     * Prints out the list of aglets
     */
    void list(PrintStream p) throws Exception {
	Enumeration e = aglets.keys();
	while(e.hasMoreElements()) {
	    String k = (String)e.nextElement();
	    Item item = (Item)aglets.get(k);
	    p.println(k + " [" + item.proxy.getAgletClassName() + "] " + item.text);
	}
    }
}

class Item {
    AgletProxy proxy;
    String text = "";

    Item(AgletProxy proxy) {
	this.proxy = proxy;
    }

    void setText(String t) {
	text = t;
    }

    /**
     * Executs the command given as arguments to the aglet instance
     */
    public void command(String mtd, StringTokenizer st) throws Exception {
	int c = st.countTokens();
	
	if ("clone".equalsIgnoreCase(mtd)) {
	    proxy.clone();
	} else if ("dispatch".equalsIgnoreCase(mtd)) {
	    if (st.hasMoreTokens()) {
		proxy.dispatch(new URL(st.nextToken()));
	    } else {
		new Error("aglet dispatch URL");
	    }
	} else if ("dialog".equalsIgnoreCase(mtd)) {
	    proxy.sendAsyncMessage(new Message("dialog"));
	} else if ("dispose".equalsIgnoreCase(mtd)) {
	    proxy.dispose();
	} else if ("property".equalsIgnoreCase(mtd)) {
	    System.out.println(toString());
	} else {
	    System.out.println("unknown method " + mtd);
	}
    }

    /**
     *
     */
    public String toString() {
	StringBuffer all = new StringBuffer();
	try {
	    AgletInfo info = proxy.getAgletInfo();
	    all.append(info.getPrivilegeName() + " Aglet" + "\n");
	    all.append(info.getAgletID() + "\n");
	    all.append(info.getAuthorityName() + "\n");
	    all.append(new Date(info.getCreationTime()));
	    all.append("\n" + info.getAgletClassName() + "\n");
	    all.append((proxy.getCodeBase() == null) ?
		       "Local host" : info.getCodeBase().toString());
	    all.append(info.getAPIMajorVersion() + '.' +
		       info.getAPIMinorVersion());

	} catch (InvalidAgletException ex) {
	    all.append("Unavailable");
	}
	return all.toString();
    }

    public boolean equals(Object obj) {
	return proxy == obj;
    }
}
